/*
 * file AuthenticationDemo2.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM - you are allowed to copy, modify and 
 * redistribute this file as part of any program that interfaces with 
 * IBM Rational CM API.
 *
 * com.ibm.rational.stp.client.samples.cc.AuthenticationDemo
 *
 * (C) Copyright IBM Corporation 2010.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.cmapi.samples.cc;

import java.util.ArrayList;
import java.util.List;

import javax.wvcm.ProviderFactory;
import javax.wvcm.WvcmException;
import javax.wvcm.ProviderFactory.Callback;
import javax.wvcm.WvcmException.ReasonCode;

import com.ibm.rational.wvcm.stp.StpProvider;
import com.ibm.rational.wvcm.stp.StpProvider.Domain;
import com.ibm.rational.wvcm.stp.StpProvider.StpCallback;
import com.ibm.rational.wvcm.stp.cc.CcProvider;
import com.ibm.rational.wvcm.stp.cc.CcProvider.CcAuthentication;

/**
 * Demonstrate advanced CM Server authentication.  It demonstrates the following
 * advanced authentication features:
 * <ul>
 * <li>Use of the StpProvider.StpCallback authentication callback interface for
 *     authentication in multiple domains
 * <li>Use of the CcCallback authentication callback interface for
 *     ClearCase-specific authentication features including
 *     ClearCase primary group and group list
 * <li>Support for HTTP proxy authentication
 * <li>HTTP proxy host and port specification
 * </ul>
 */
public class AuthenticationDemo2 extends CcDemoBase {

    public class DemoCallback2 implements StpCallback {

        /* (non-Javadoc)
         * @see javax.wvcm.ProviderFactory.Callback#getAuthentication(java.lang.String, int)
         */
        public Authentication
        getAuthentication(String realm, int retryCount) throws WvcmException {
            throw new IllegalStateException("Should be calling getAuthenticationEx()");
        }

        /* (non-Javadoc)
         * @see com.ibm.rational.wvcm.stp.StpProvider.StpCallback#getAuthenticationEx(com.ibm.rational.wvcm.stp.StpProvider.Domain, java.lang.String, int, com.ibm.rational.wvcm.stp.StpProvider, javax.wvcm.WvcmException)
         */
        public Authentication
        getAuthenticationEx(
                Domain domain,
                String realm,
                int retryCount,
                StpProvider provider,
                WvcmException failure
        ) throws WvcmException {

            if (retryCount > 1) {
                if (failure != null) {
                    throw failure;
                } else {
                    throw new WvcmException("Max login retry count exceeded", ReasonCode.UNAUTHORIZED);
                }
            }

            /* Supply the credentials for the specified domain.
             * "NONE" implies we're doing HTTP proxy authentication.
             */
            switch(domain) {
            case CLEAR_CASE:
                trace("Getting ClearCase credentials for: " + realm);
                return getClearCaseCreds();
            case CLEAR_QUEST:
                trace("Getting ClearQuest credentials for: " + realm);
                return getClearQuestCreds();
            case NONE:
                trace("Getting HTTP proxy credentials for: " + realm);
                return getHttpProxyCreds();
            case INVALID:
            default:
                throw new IllegalStateException("Unexpected login domain: " + domain);
            }
        }

        /**
         * Get the ClearCase credentials for the CM server, including ClearCase
         * primary group and group list.  Note the use of the ClearCase-specific
         * authentication interface "CcAuthentication".
         */
        private Authentication getClearCaseCreds() {
            return new CcAuthentication() {
                /* (non-Javadoc)
                 * @see javax.wvcm.ProviderFactory.Callback.Authentication#loginName()
                 */
                public String loginName() {
                    return props().getLoginNameAndDomain();
                }
                /* (non-Javadoc)
                 * @see javax.wvcm.ProviderFactory.Callback.Authentication#password()
                 */
                public String password() {
                    return props().getPassword();
                }
                /* (non-Javadoc)
                 * @see com.ibm.rational.wvcm.stp.cc.CcProvider.CcAuthentication#getPrimaryGroupName()
                 */
                public String getPrimaryGroupName() {
                    return props().getPrimaryGroup();
                }
                /* (non-Javadoc)
                 * @see com.ibm.rational.wvcm.stp.cc.CcProvider.CcAuthentication#getGroupList()
                 */
                public List<String> getGroupList() {
                    //TODO: supply real group list.
                    return new ArrayList<String>();
                }
            };
        }

        /**
         * Get the ClearQuest credentials for the CM server.  These credentials
         * are only needed if you are performing ClearQuest-enabled
         * UCM operations on the CM Server.
         */
        private Authentication getClearQuestCreds() {
            return new Authentication() {
                /* (non-Javadoc)
                 * @see javax.wvcm.ProviderFactory.Callback.Authentication#loginName()
                 */
                public String loginName() {
                    return props().getCqLoginName();
                }
                /* (non-Javadoc)
                 * @see javax.wvcm.ProviderFactory.Callback.Authentication#password()
                 */
                public String password() {
                    return props().getCqPassword();
                }
            };
        }

        /**
         * Get the credentials for the HTTP proxy.  These credentials are only
         * needed if you are connecting to a CM Server through an authenticating
         * HTTP proxy.
         */
        private Authentication getHttpProxyCreds() {
            return new Authentication() {
                /* (non-Javadoc)
                 * @see javax.wvcm.ProviderFactory.Callback.Authentication#loginName()
                 */
                public String loginName() {
                    return props().getHttpProxyLoginName();
                }
                /* (non-Javadoc)
                 * @see javax.wvcm.ProviderFactory.Callback.Authentication#password()
                 */
                public String password() {
                    return props().getHttpProxyPassword();
                }
            };
        }
    }

    /**
     * Demostrate advanced CM API authentication features including specifying
     * the HTTP proxy host and port numbers.
     */
    public void testAdvancedAuthentication() throws Exception {

        // Instantiate our own authentication callback.
        Callback callback = new DemoCallback2();

        // Instantiate our own CcProvider so we can specify our own callback.
        StpProvider stpProvider =
            (StpProvider) ProviderFactory.createProvider(
                CcProvider.NETWORK_PROVIDER_CLASS, callback);
        CcProvider ccProvider = stpProvider.ccProvider();
        ccProvider.setServerUrl(props().getCmServerUrl());

        // Set forward HTTP proxy host and port if specified in the user's
        // demo properties file.
        String httpProxyHost = props().getHttpProxyHost();
        String httpProxyPort = props().getHttpProxyPort();
        if (httpProxyHost != null && httpProxyPort != null) {
            ccProvider.setHttpProxy(httpProxyHost, httpProxyPort);
        }

        // Now perform an operation that requires a CM server connection.
        // This will cause our authentication callback to be invoked.
        ccProvider.doGetDefaultCcRegistryRegion(null);
    }
}
